@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
@@ -115,13 +115,14 @@ def get_new_os_window_trigger(opts):
     return new_os_window_trigger
 
 
-def _run_app(opts, args):
+def _run_app(opts, args, bad_lines=()):
     new_os_window_trigger = get_new_os_window_trigger(opts)
     if is_macos and opts.macos_custom_beam_cursor:
         set_custom_ibeam_cursor()
     if not is_wayland and not is_macos:  # no window icons on wayland
         with open(logo_data_file, 'rb') as f:
             set_default_window_icon(f.read(), 256, 256)
+    load_shader_programs.use_selection_fg = opts.selection_foreground is not None
     with cached_values_for(run_app.cached_values_name) as cached_values:
         with startup_notification_handler(extra_callback=run_app.first_window_callback) as pre_show_callback:
             window_id = create_os_window(
@@ -131,18 +132,20 @@ def _run_app(opts, args):
                     args.cls or appname, load_all_shaders)
         boss = Boss(window_id, opts, args, cached_values, new_os_window_trigger)
         boss.start()
+        if bad_lines:
+            boss.show_bad_config_lines(bad_lines)
         try:
             boss.child_monitor.main_loop()
         finally:
             boss.destroy()
 
 
-def run_app(opts, args):
+def run_app(opts, args, bad_lines=()):
     set_scale(opts.box_drawing_scale)
     set_options(opts, is_wayland, args.debug_gl, args.debug_font_fallback)
     set_font_family(opts, debug_font_matching=args.debug_font_fallback)
     try:
-        _run_app(opts, args)
+        _run_app(opts, args, bad_lines)
     finally:
         free_font_data()  # must free font data before glfw/freetype/fontconfig/opengl etc are finalized
 
@@ -152,7 +155,7 @@ run_app.first_window_callback = lambda window_handle: None
 run_app.initial_window_size_func = initial_window_size_func
 
 
-def ensure_osx_locale():
+def ensure_macos_locale():
     # Ensure the LANG env var is set. See
     # https://github.com/kovidgoyal/kitty/issues/90
     from .fast_data_types import cocoa_get_lang
@@ -214,7 +217,7 @@ def _main():
     except AttributeError:
         pass  # python compiled without threading
     if is_macos:
-        ensure_osx_locale()
+        ensure_macos_locale()
     try:
         locale.setlocale(locale.LC_ALL, '')
     except Exception:
@@ -231,7 +234,7 @@ def _main():
     rpath = os.path.dirname(kitty_exe())
     items = frozenset(os.environ['PATH'].split(os.pathsep))
     if rpath and rpath not in items:
-        os.environ['PATH'] += os.pathsep + rpath
+        os.environ['PATH'] = rpath + os.pathsep + os.environ.get('PATH', '')
 
     args = sys.argv[1:]
     if is_macos and os.environ.pop('KITTY_LAUNCHED_BY_LAUNCH_SERVICES', None) == '1':
@@ -261,12 +264,13 @@ def _main():
             talk_to_instance(args)
             return
     init_glfw(args.debug_keyboard)  # needed for parsing native keysyms
-    opts = create_opts(args)
+    bad_lines = []
+    opts = create_opts(args, accumulate_bad_lines=bad_lines)
     setup_environment(opts, args)
     try:
         with setup_profiling(args):
             # Avoid needing to launch threads to reap zombies
-            run_app(opts, args)
+            run_app(opts, args, bad_lines)
     finally:
         glfw_terminate()
 
